@@ -23,7 +23,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_tar_filenames.c,v 1.8 2008/01/01 22:28:04 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/test/test_tar_filenames.c,v 1.10 2008/09/01 05:38:33 kientzle Exp $");
 
 /*
  * Exercise various lengths of filenames in tar archives,
@@ -40,19 +40,22 @@ test_filename(const char *prefix, int dlen, int flen)
 	struct archive_entry *ae;
 	struct archive *a;
 	size_t used;
-	size_t prefix_length = 0;
-	unsigned i = 0;
+	char *p;
+	int i;
 
+	p = filename;
 	if (prefix) {
 		strcpy(filename, prefix);
-		i = prefix_length = strlen(prefix);
+		p += strlen(p);
 	}
-	for (; i < prefix_length + dlen; i++)
-		filename[i] = 'a';
-	filename[i++] = '/';
-	for (; i < prefix_length + dlen + flen + 1; i++)
-		filename[i] = 'b';
-	filename[i++] = '\0';
+	if (dlen > 0) {
+		for (i = 0; i < dlen; i++)
+			*p++ = 'a';
+		*p++ = '/';
+	}
+	for (i = 0; i < flen; i++)
+		*p++ = 'b';
+	*p = '\0';
 
 	strcpy(dirname, filename);
 
@@ -98,10 +101,10 @@ test_filename(const char *prefix, int dlen, int flen)
 
 	/* Close out the archive. */
 	assertA(0 == archive_write_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assertA(0 == archive_write_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_write_finish(a);
+#else
+	assertA(0 == archive_write_finish(a));
 #endif
 
 	/*
@@ -114,7 +117,7 @@ test_filename(const char *prefix, int dlen, int flen)
 
 	/* Read the file and check the filename. */
 	assertA(0 == archive_read_next_header(a, &ae));
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 	skipping("Leading '/' preserved on long filenames");
 #else
 	assertEqualString(filename, archive_entry_pathname(ae));
@@ -130,7 +133,7 @@ test_filename(const char *prefix, int dlen, int flen)
 	 * here.
 	 */
 	assertA(0 == archive_read_next_header(a, &ae));
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 	skipping("Trailing '/' preserved on dirnames");
 #else
 	assertEqualString(dirname, archive_entry_pathname(ae));
@@ -138,7 +141,7 @@ test_filename(const char *prefix, int dlen, int flen)
 	assert((S_IFDIR | 0755) == archive_entry_mode(ae));
 
 	assertA(0 == archive_read_next_header(a, &ae));
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 	skipping("Trailing '/' added to dir names");
 #else
 	assertEqualString(dirname, archive_entry_pathname(ae));
@@ -148,10 +151,10 @@ test_filename(const char *prefix, int dlen, int flen)
 	/* Verify the end of the archive. */
 	assert(1 == archive_read_next_header(a, &ae));
 	assert(0 == archive_read_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_read_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_read_finish(a);
+#else
+	assert(0 == archive_read_finish(a));
 #endif
 }
 
@@ -160,15 +163,22 @@ DEFINE_TEST(test_tar_filenames)
 	int dlen, flen;
 
 	/* Repeat the following for a variety of dir/file lengths. */
-	for (dlen = 40; dlen < 60; dlen++) {
-		for (flen = 40; flen < 60; flen++) {
+	for (dlen = 45; dlen < 55; dlen++) {
+		for (flen = 45; flen < 55; flen++) {
+			test_filename(NULL, dlen, flen);
+			test_filename("/", dlen, flen);
+		}
+	}
+
+	for (dlen = 0; dlen < 140; dlen += 10) {
+		for (flen = 98; flen < 102; flen++) {
 			test_filename(NULL, dlen, flen);
 			test_filename("/", dlen, flen);
 		}
 	}
 
 	for (dlen = 140; dlen < 160; dlen++) {
-		for (flen = 90; flen < 110; flen++) {
+		for (flen = 95; flen < 105; flen++) {
 			test_filename(NULL, dlen, flen);
 			test_filename("/", dlen, flen);
 		}
